#!/usr/bin/env bash
source ../multi-host.conf
testCase=$1
workDir=$(pwd)
logDir=$workDir/logs
hostIP=$IP4_A1
hubIP=$IP4_B1
testLog=$logDir/remote-register.log
hostSh=A.sh
hubSh=B.sh

echo "TEST REMOTE REGISTER START" >$testLog

LOG () {
    echo $1; echo $1 >>$testLog
}

run_hub_cmd () {
    ssh $hubIP "$1" 2>&1
}

run_scp_to_hub () {
    scp $1 $hubIP:$workDir/$1 2>&1
}

set_hub_trust_anchor () {
    LOG "set the trust anchor to $1 on the hub."
    anchor=$(sh $hostSh -a $1)
    echo $anchor > anchor.cert
    run_scp_to_hub anchor.cert && rm anchor.cert
}

run_host_ping_server() {
    ndnpingserver $1 >>$testLog 2>&1 &
}

end_test() {
    scp $hubIP:$workDir/hub.nfd.log hub.nfd.log 2>&1
    scp $hubIP:$workDir/hub.test.log hub.test.log 2>&1
    cat host.nfd.log hub.nfd.log > nfd.log
    cat host.test.log hub.test.log > test.log
    cat nfd.log test.log >> $testLog

    bash $hostSh -c && run_hub_cmd "cd $workDir && bash $hubSh -c"
    run_hub_cmd "[[ -d $workDir ]] && sudo rm -rf $workDir"
    rm *.log

    [[ $# -gt 0 ]] && LOG "$1" && exit 1 || exit 0
}

inspect_hub_rib_exist() {
    record=$(run_hub_cmd "nfd-status -r" | grep "$1 route")
    if [[ -z "$record" ]]; then
        end_test "Fail test $2"
    fi
}

inspect_hub_rib_none() {
    record=$(run_hub_cmd "nfd-status -r" | grep "$1 route")
    if [[ ! -z "$record" ]]; then
        end_test "Fail test $2. the matched record: $record"
    fi
}

run_test() {
    faceUri=udp4://$hubIP:6363
    nfdc create $faceUri >> $testLog

    defer_time=5 && test_case=0

    test_case=`expr $test_case + 1` && LOG "run test case: $test_case" #1
    run_host_ping_server ndn:/Z/A/G && p1=$! && sleep $defer_time
    inspect_hub_rib_none /Z/A "$test_case: route of /Z/A exists."

    test_case=`expr $test_case + 1` && LOG "run test case: $test_case" #2
    nfdc register ndn:/localhop/nfd $faceUri >> $testLog
    # the route of /Z/A is expected to exist. But due to BUG 2413, it will not
    inspect_hub_rib_none /Z/A "$test_case: route of /Z/A does not exist."

    test_case=`expr $test_case + 1` && LOG "run test case: $test_case" #3
    run_host_ping_server ndn:/Z/A/H && p2=$! && sleep $defer_time
    inspect_hub_rib_exist /Z/A "$test_case: route of /Z/A does not exist."

    test_case=`expr $test_case + 1` && LOG "run test case: $test_case" #4
    sudo kill $p1 && sleep $defer_time
    inspect_hub_rib_exist /Z/A "$test_case: route of /Z/A was unregistered."

    test_case=`expr $test_case + 1` && LOG "run test case: $test_case" #5
    sudo kill $p2 && sleep $defer_time
    inspect_hub_rib_none /Z/A "$test_case: route of /Z/A still exists."

    test_case=`expr $test_case + 1` && LOG "run test case: $test_case" #6
    run_host_ping_server ndn:/Z/A/B/C/D && p3=$! && sleep $defer_time
    inspect_hub_rib_none /Z/A "$test_case: route of /Z/A exists."
    inspect_hub_rib_exist /Z/A/B/C "$test_case: route of /Z/A/B/C does not exist."

    test_case=`expr $test_case + 1` && LOG "run test case: $test_case" #7
    sudo kill $p3 && run_host_ping_server ndn:/Z/A/B/C/D/E && sleep $defer_time
    inspect_hub_rib_none /Z/A "$test_case: route of /Z/A exists."
    inspect_hub_rib_none /Z/A/B/C "$test_case: route of /Z/A/B/C exists."
    inspect_hub_rib_exist /Z/A/B/C/D/E "$test_case: route of /Z/A/B/C/D/E does not exist."

    test_case=`expr $test_case + 1` && LOG "run test case: $test_case" #8
    sleep 80
    inspect_hub_rib_exist /Z/A/B/C/D/E "$test_case: route of /Z/A/B/C/D/E does not exist."

    test_case=`expr $test_case + 1` && LOG "run test case: $test_case" #9
    run_host_ping_server ndn:/Z/F/I && sleep $defer_time
    inspect_hub_rib_none /Z/F "$test_case: route of /Z/F exist."
}

LOG "set environment in the host."
bash $hostSh -c -p

LOG "set environment in the hub."
run_hub_cmd "[[ -d $workDir ]] || mkdir -p $workDir"
run_scp_to_hub $hubSh
set_hub_trust_anchor ndn:/Z/A
run_hub_cmd "cd $workDir && bash $hubSh -c -p"

LOG "run test"
run_test && end_test
